% matlab demo lecture 9
% frequency domain parameterization
% April 26 2010
%
clc
%close all
clear all

nrz_v = [0 0 0 0 2 2]; %# zeros
nrp_v = [1 2 3 4 4 5]; %# poles

z1 = -30;
z2 = -40;
p1 = -3;
p2 = -10;
p3 = -80;
p4 = -100;
k = abs(z1 * z2) / abs(p1 * p2 * p3 * p4);
sys = zpk([z1 z2], [p1 p2 p3 p4], 1);

T = 10;
dt = 1e-3;
t = (0 : dt : T)';
N = length(t);
%ut = randn(N, 1);
%save ut ut
load ut
yt = lsim(sys, ut, t);
%yt = yt + randn(N, 1) * std(yt) * 0.5;

f = logspace(log10(0.1), log10(500), 7000)';
s = 2 * pi * f * j;
H = ((s - z1) .* (s - z2)) ./ ((s - p1) .* (s - p2) .* (s - p3) .* (s - p4));

figure(1)
subplot(211)
plot(t, ut, 'r', 'linewidth', 2), hold on
set(gca, 'fontsize', 16)
ylabel('Input')
subplot(212)
plot(t, yt, 'r', 'linewidth', 2), hold on
set(gca, 'fontsize', 16)
ylabel('Output')
xlabel('Time [s]')
set(gcf, 'position', [1059         176         640         871], 'color', 'k')

figure(2)
subplot(211)
loglog(f, abs(H), 'r', 'linewidth', 2), hold on
set(gca, 'fontsize', 16)
ylabel('Gain [-]')

subplot(212)
semilogx(f, unwrap(angle(H)) * 180 / pi, 'r', 'linewidth', 2), hold on
set(gca, 'fontsize', 16)
ylabel('Phase [deg]')
set(gcf, 'position', [1059         176         640         871], 'color', 'k')

options=optimset('display','iter','maxfunevals',500,'maxiter',1000,'tolfun',1e-6,'diffminchange',1e-3,'diffmaxchange',1e0,'tolx',1e-4);

Zeros = [z1 z2];
Poles = [p1 p2 p3 p4 mean([p1 p2 p3 p4])];
randpar = ones(1, 10) .* (1 + 0.5 * rand(1, 10));

nrcond = length(nrz_v);
for cond = 1 : nrcond

    nrz = nrz_v(cond);
    nrp = nrp_v(cond);

    Ps = [Zeros(1 : nrz) Poles(1 : nrp) 1];
    P0 = randpar(1 : length(Ps));
    
    [Popt,Resnorm,res,Exitflag,Output,Lambda,J] = lsqnonlin('fitfun_freq',P0, [], [], options, Ps, H, f);
    P = Popt .* Ps;

    N = length(res);
    Hess=(J'*J)/N;
    E=res'*res/N;
    CovP=full(E*inv(Hess)/N);
    stdxx = abs(sqrt(diag(abs(CovP))));
    diagdom = not(max(abs(CovP), [], 1)>diag(CovP)');

    sem = zeros(8, 1);
    ddom = zeros(8, 1);
    if length(P) == 2
        z1m = [];
        z2m = [];
        p1m = P(1);
        p2m = [];
        p3m = [];
        p4m = [];
        p5m = [];
        km = P(2);
        sem([3 8]) = stdxx;
        ddom([3 8]) = diagdom;
        H_mod = km  ./ (s - p1m);
        sys_mod = zpk([z1m z2m], [p1m p2m p3m p4m p5m], km);
        kmtot = abs(km / p1m);
    end
    if length(P) == 3
        z1m = [];
        z2m = [];
        p1m = P(1);
        p2m = P(2);
        p3m = [];
        p4m = [];
        p5m = [];
        km = P(3);
        sem([3:4 8]) = stdxx;
        ddom([3:4 8]) = diagdom;
        H_mod = km ./ ((s - p1m) .* (s - p2m));
        sys_mod = zpk([z1m z2m], [p1m p2m p3m p4m p5m], km);
        kmtot = abs(km / (p1m * p2m));
    end
    if length(P) == 4
        z1m = [];
        z2m = [];
        p1m = P(1);
        p2m = P(2);
        p3m = P(3);
        p4m = [];
        p5m = [];
        km = P(4);
        sem([3:5 8]) = stdxx;
        ddom([3:5 8]) = diagdom;
        H_mod = km ./ ((s - p1m) .* (s - p2m) .* (s - p3m));
        sys_mod = zpk([z1m z2m], [p1m p2m p3m p4m p5m], km);
        kmtot = abs(km / (p1m * p2m * p3m));
    end
    if length(P) == 5
        z1m = [];
        z2m = [];
        p1m = P(1);
        p2m = P(2);
        p3m = P(3);
        p4m = P(4);
        p5m = [];
        km = P(5);
        sem([3:6 8]) = stdxx;
        ddom([3:6 8]) = diagdom;
        H_mod = km ./ ((s - p1m) .* (s - p2m) .* (s - p3m) .* (s - p4m));
        sys_mod = zpk([z1m z2m], [p1m p2m p3m p4m p5m], km);
        kmtot = abs(km / (p1m * p2m * p3m * p4m));
    end
    if length(P) == 7
        z1m = P(1);
        z2m = P(2);
        p1m = P(3);
        p2m = P(4);
        p3m = P(5);
        p4m = P(6);
        p5m = [];
        km = P(7);
        sem([1:6 8]) = stdxx;
        ddom([1:6 8]) = diagdom;
        H_mod = km * ((s - z1m) .* (s - z2m)) ./ ((s - p1m) .* (s - p2m) .* (s - p3m) .* (s - p4m));
        sys_mod = zpk([z1m z2m], [p1m p2m p3m p4m p5m], km);
        kmtot = abs(km * (z1m * z2m) / (p1m * p2m * p3m * p4m));
    end
    if length(P) == 8
        z1m = P(1);
        z2m = P(2);
        p1m = P(3);
        p2m = P(4);
        p3m = P(5);
        p4m = P(6);
        p5m = P(7);
        km = P(8);
        sem(1:8) = stdxx;
        ddom(1:8) = diagdom;
        H_mod = km * ((s - z1m) .* (s - z2m)) ./ ((s - p1m) .* (s - p2m) .* (s - p3m) .* (s - p4m) .* (s - p5m));
        sys_mod = zpk([z1m z2m], [p1m p2m p3m p4m p5m], km);
        kmtot = abs(km * (z1m * z2m) / (p1m * p2m * p3m * p4m * p5m));
    end
    yt_mod = lsim(sys_mod, ut, t);

    figure(1)
    subplot(212)
    plot(t, yt_mod, 'b:', 'linewidth', 2)
    legend('system', 'model')

    figure(2)
    subplot(211)
    loglog(f, abs(H_mod), 'b:', 'linewidth', 2)

    subplot(212)
    semilogx(f, unwrap(angle(H_mod)) * 180 / pi, 'b:', 'linewidth', 2)

    VAF = 100 * (1 - sum(abs(yt_mod(3/dt:end) - yt(3/dt:end)).^2) ./ sum(abs(yt(3/dt:end)).^2));
    disp(' ')
    disp(['VAF: ' num2str(VAF)])

    showpar = {
        'par', 'true', 'estimated', 'sem', 'domin'
        'z1', z1,  z1m, sem(1), ddom(1)
        'z2', z2,  z2m, sem(2), ddom(2)
        'p1', p1,  p1m, sem(3), ddom(3)
        'p2', p2,  p2m, sem(4), ddom(4)
        'p3', p3,  p3m, sem(5), ddom(5)
        'p4', p4,  p4m, sem(6), ddom(6)
        'p5', '-', p5m, sem(7), ddom(7)
        'k' , k,   kmtot , sem(8), ddom(8)
        };

    disp(showpar)

    figure(3)
    subplot(311)
    plot(length(P0) - 1, VAF, 'r*', 'markersize', 20, 'linewidth', 3), hold on
    set(gca, 'fontsize', 16)
    ylabel('VAF')

    subplot(312)
    plot(length(P0) - 1, log(mean(sem)), 'r*', 'markersize', 20, 'linewidth', 3), hold on
    set(gca, 'fontsize', 16)
    ylabel('SEM Average (log)')

    subplot(313)
    plot(length(P0) - 1, sum(diagdom), 'r*', 'markersize', 20, 'linewidth', 3), hold on
    set(gca, 'fontsize', 16, 'ytick',[0 : 5])
    ylabel('# diag. dom. parameters')
    xlabel('# Model Parameters (poles + zeros)')
    set(gcf, 'position', [1059         176         640         871], 'color', 'k')

%     pause

end

% Conclusions
% Inspect different model structures (orders) to select the best
% VAF and SEM together are needed to select
% Cross-covariance is not exclusive to select: surplus sway-off parameters
% may act on their own.
% Cross-covariance larger in time domain wrt frequency domain: less
% influnece from neighbouring behavior in f-domain